@@ -1,12 +1,17 @@
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-.
-"""Provides general routines for the ACE space weather instruments
-"""
+# Full license can be found in License.md
+# Full author list can be found in .zenodo.json file
+# DOI:10.5281/zenodo.3986138
+# ----------------------------------------------------------------------------
+"""Provides general routines for the ACE space weather instruments."""
 
 import datetime as dt
 import numpy as np
 import os
 import pandas as pds
 import requests
+import warnings
 
 import pysat
 
@@ -14,11 +19,11 @@ logger = pysat.logger
 
 
 def acknowledgements():
-    """Returns acknowledgements for the specified ACE instrument
+    """Define the acknowledgements for the specified ACE instrument.
 
     Returns
     -------
-    ackn : string
+    ackn : str
         Acknowledgements for the ACE instrument
 
     """
@@ -33,16 +38,16 @@ def acknowledgements():
 
 
 def references(name):
-    """Returns references for the specified ACE instrument
+    """Define the references for the specified ACE instrument.
 
     Parameters
     ----------
-    name : string
+    name : str
         Instrument name of the ACE instrument
 
     Returns
     -------
-    ref : string
+    ref : str
         Reference for the ACE instrument paper
 
     """
@@ -77,7 +82,7 @@ def references(name):
 
 
 def clean(inst):
-    """Common aspects of the ACE space weather data cleaning
+    """Clean the common aspects of the ACE space weather data.
 
     Parameters
     ----------
@@ -113,26 +118,26 @@ def clean(inst):
     return max_status
 
 
-def list_files(name='', tag='', inst_id='', data_path='', format_str=None):
-    """Return a Pandas Series of every file for ACE data
+def list_files(name, tag='', inst_id='', data_path='', format_str=None):
+    """List the local ACE data files.
 
     Parameters
     ----------
     name : str
-        ACE Instrument name. (default='')
+        ACE Instrument name.
     tag : str
-        Denotes type of file to load. (default='')
+        ACE Instrument tag. (default='')
     inst_id : str
         Specifies the ACE instrument ID. (default='')
     data_path : str
         Path to data directory. (default='')
-    format_str : string or NoneType
+    format_str : str or NoneType
         User specified file format.  If None is specified, the default
         formats associated with the supplied tags are used. (default=None)
 
     Returns
     -------
-    pysat.Files.from_os : pysat.utils.files.Files
+    files : pysat.Files
         A class containing the verified available files
 
     Note
@@ -149,20 +154,19 @@ def list_files(name='', tag='', inst_id='', data_path='', format_str=None):
     return files
 
 
-def download(date_array, name='', tag='', inst_id='', data_path='', now=None):
-    """Routine to download ACE Space Weather data
+def download(date_array, name, tag='', inst_id='', data_path='', now=None):
+    """Download the requested ACE Space Weather data.
 
     Parameters
     ----------
     date_array : array-like
         Array of datetime values
     name : str
-        ACE Instrument name. (default='')
+        ACE Instrument name.
     tag : str
-        Denotes type of file to load. Accepted types are 'realtime' and
-        'historic'. (default='')
+        ACE Instrument tag. (default='')
     inst_id : str
-        Specifies the ACE instrument ID. (default='')
+        ACE instrument ID. (default='')
     data_path : str
         Path to data directory. (default='')
     now : dt.datetime or NoneType
@@ -179,6 +183,7 @@ def download(date_array, name='', tag='', inst_id='', data_path='', now=None):
     - File requested not available on server
 
     """
+
     # Ensure now is up-to-date, if desired
     if now is None:
         now = dt.datetime.utcnow()
@@ -258,7 +263,7 @@ def download(date_array, name='', tag='', inst_id='', data_path='', now=None):
 
 
 def common_metadata():
-    """Provides common metadata information for all ACE instruments
+    """Define the common metadata information for all ACE instruments.
 
     Returns
     -------
@@ -296,7 +301,11 @@ def common_metadata():
 
 
 def load_csv_data(fnames, read_csv_kwargs=None):
-    """Load CSV data from a list of files into a single DataFrame
+    """Load CSV data from a list of files into a single DataFrame.
+
+    .. deprecated:: 0.0.5
+        `load_csv_data` will be removed in pysatSpaceWeather 0.0.6+, as it has
+        been moved to `pysat.instruments.methods.general` as of pysat 3.0.1.
 
     Parameters
     ----------
@@ -312,9 +321,15 @@ def load_csv_data(fnames, read_csv_kwargs=None):
 
     See Also
     --------
-    pds.read_csv
+    pds.read_csv, pysat.instruments.methods.general.load_csv_data
 
     """
+
+    warnings.warn("".join(["Moved to pysat.instruments.methods.general.",
+                           "load_csv_data in pysat version 3.0.1. This method ",
+                           "will be removed at the 0.0.6+ release."]),
+                  DeprecationWarning)
+
     # Ensure the filename input is array-like
     fnames = np.asarray(fnames)
     if fnames.shape == ():
@@ -331,3 +346,75 @@ def load_csv_data(fnames, read_csv_kwargs=None):
 
     data = pds.DataFrame() if len(fdata) == 0 else pds.concat(fdata, axis=0)
     return data
+
+
+def ace_swepam_hourly_omni_norm(as_inst, speed_key='sw_bulk_speed',
+                                dens_key='sw_proton_dens',
+                                temp_key='sw_ion_temp'):
+    """Normalize ACE SWEPAM variables as described in the OMNI processing _[1].
+
+    Parameters
+    ----------
+    as_inst : pysat.Instrument
+        pysat Instrument object with ACE SWEPAM data.
+    speed_key : str
+        Data key for bulk solar wind speed data in km/s
+        (default='sw_bulk_speed')
+    dens_key : str
+        Data key for solar wind proton density data in P/cm^3
+        (default='sw_proton_dens')
+    temp_key : str
+        Data key for solar wind ion temperature data in K
+        (default='sw_ion_temp')
+
+    References
+    ----------
+    [1] https://omniweb.gsfc.nasa.gov/html/omni_min_data.html
+
+    """
+
+    # Check the input to make sure all the necessary data variables are present
+    for var in [speed_key, dens_key, temp_key]:
+        if var not in as_inst.variables:
+            raise ValueError('instrument missing variable: {:}'.format(var))
+
+    # Let yt be the fractional years since 1998.0
+    yt = np.array([pysat.utils.time.datetime_to_dec_year(itime) - 1998.0
+                   for itime in as_inst.index])
+
+    # Get the masks for the different velocity limits
+    ilow = as_inst[speed_key] < 395
+    imid = (as_inst[speed_key] >= 395) & (as_inst[speed_key] <= 405)
+    ihigh = as_inst[speed_key] > 405
+
+    # Calculate the normalized plasma density
+    norm_n = np.array(as_inst[dens_key])
+    norm_n[ilow] *= (0.925 + 0.0039 * yt[ilow])
+    norm_n[imid] *= (74.02 - 0.164 * as_inst[speed_key][imid]
+                     + 0.0171 * as_inst[speed_key][imid] * yt[imid]
+                     - 6.72 * yt[imid]) / 10.0
+    norm_n[ihigh] *= (0.761 + 0.0210 * yt[ihigh])
+
+    # Normalize the temperature
+    norm_t = np.power(10.0, -0.069 + 1.024 * np.log10(as_inst[temp_key]))
+
+    # Update the instrument data
+    as_inst['sw_proton_dens_norm'] = pds.Series(norm_n, index=as_inst.index)
+    as_inst['sw_ion_temp_norm'] = pds.Series(norm_t, index=as_inst.index)
+
+    # Add meta data
+    for dkey in [dens_key, temp_key]:
+        nkey = '{:s}_norm'.format(dkey)
+        meta_dict = {}
+
+        for mkey in as_inst.meta[dkey].keys():
+            if mkey == as_inst.meta.labels.notes:
+                meta_dict[mkey] = ''.join([
+                    'Normalized for hourly OMNI as described in ',
+                    'https://omniweb.gsfc.nasa.gov/html/omni_min_data.html'])
+            elif mkey != "children":
+                meta_dict[mkey] = as_inst.meta[dkey, mkey]
+
+        as_inst.meta[nkey] = meta_dict
+
+    return
