@@ -1,10 +1,15 @@
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-.
-"""Provides default routines for solar wind and geospace indices
+# Full license can be found in License.md
+# Full author list can be found in .zenodo.json file
+# DOI:10.5281/zenodo.3986138
+# ----------------------------------------------------------------------------
+"""Provides routines to support the geomagnetic indeces, Kp and Ap."""
 
-"""
-
-import pandas as pds
+import datetime as dt
 import numpy as np
+from packaging.version import Version
+import pandas as pds
 
 import pysat
 
@@ -15,14 +20,14 @@ import pysatSpaceWeather as pysat_sw
 # Instrument utilities
 
 def acknowledgements(name, tag):
-    """Returns acknowledgements for space weather dataset
+    """Define the acknowledgements for the geomagnetic data sets.
 
     Parameters
     ----------
-    name : string
-        Name of space weather index, eg, dst, f107, kp
-    tag : string
-        Tag of the space waether index
+    name : str
+        Instrument name of space weather index, accepts 'kp' or 'ap'.
+    tag : str
+        Instrument tag.
 
     """
     swpc = ''.join(['Prepared by the U.S. Dept. of Commerce, NOAA, Space ',
@@ -35,14 +40,14 @@ def acknowledgements(name, tag):
 
 
 def references(name, tag):
-    """Returns references for space weather dataset
+    """Define the references for the geomagnetic data sets.
 
     Parameters
     ----------
-    name : string
-        Name of space weather index, eg, dst, f107, kp
-    tag : string
-        Tag of the space waether index
+    name : str
+        Instrument name of space weather index, accepts kp or ap.
+    tag : str
+        Instrument tag.
 
     """
 
@@ -73,7 +78,7 @@ def references(name, tag):
 
 
 def initialize_kp_metadata(meta, data_key, fill_val=-1):
-    """ Initialize the Kp meta data using our knowledge of the index
+    """Initialize the Kp meta data using our knowledge of the index.
 
     Parameters
     ----------
@@ -99,19 +104,28 @@ def initialize_kp_metadata(meta, data_key, fill_val=-1):
 # Common custom functions
 
 
-def convert_3hr_kp_to_ap(kp_inst):
-    """ Calculate 3 hour ap from 3 hour Kp index
+def convert_3hr_kp_to_ap(kp_inst, var_name='Kp'):
+    """Calculate 3 hour ap from 3 hour Kp index.
 
     Parameters
     ----------
     kp_inst : pysat.Instrument
-        Pysat instrument containing Kp data
+        Instrument containing Kp data
+    var_name : str
+        Variable name for the Kp data (default='Kp')
 
-    Notes
-    -----
+    Raises
+    ------
+    ValueError
+        If `var_name` is not present in `kp_inst`.
+
+    Note
+    ----
     Conversion between ap and Kp indices is described at:
     https://www.ngdc.noaa.gov/stp/GEOMAG/kp_ap.html
 
+    Assigns new data to '3hr_ap'.
+
     """
 
     # Kp are keys, where n.3 = n+ and n.6 = (n+1)-. E.g., 0.6 = 1-
@@ -127,13 +141,14 @@ def convert_3hr_kp_to_ap(kp_inst):
             return np.nan
 
     # Test the input
-    if 'Kp' not in kp_inst.data.columns:
-        raise ValueError('unable to locate Kp data')
+    if var_name not in kp_inst.variables:
+        raise ValueError('Variable name for Kp data is missing: {:}'.format(
+            var_name))
 
     # Convert from Kp to ap
-    fill_val = kp_inst.meta['Kp'][kp_inst.meta.labels.fill_val]
+    fill_val = kp_inst.meta[var_name][kp_inst.meta.labels.fill_val]
     ap_data = np.array([ap(kp) if kp != fill_val else fill_val
-                        for kp in kp_inst['Kp']])
+                        for kp in kp_inst[var_name]])
 
     # Append the output to the pysat instrument
     kp_inst['3hr_ap'] = pds.Series(ap_data, index=kp_inst.index)
@@ -155,7 +170,7 @@ def convert_3hr_kp_to_ap(kp_inst):
 
 def calc_daily_Ap(ap_inst, ap_name='3hr_ap', daily_name='Ap',
                   running_name=None):
-    """ Calculate the daily Ap index from the 3hr ap index
+    """Calculate the daily Ap index from the 3hr ap index.
 
     Parameters
     ----------
@@ -169,6 +184,11 @@ def calc_daily_Ap(ap_inst, ap_name='3hr_ap', daily_name='Ap',
         Column name for daily running average of ap, not output if None
         (default=None)
 
+    Raises
+    ------
+    ValueError
+        If `ap_name` or `daily_name` aren't present in `ap_inst`
+
     Note
     ----
     Ap is the mean of the 3hr ap indices measured for a given day
@@ -179,11 +199,11 @@ def calc_daily_Ap(ap_inst, ap_name='3hr_ap', daily_name='Ap',
     """
 
     # Test that the necessary data is available
-    if ap_name not in ap_inst.data.columns:
+    if ap_name not in ap_inst.variables:
         raise ValueError("bad 3-hourly ap column name: {:}".format(ap_name))
 
     # Test to see that we will not be overwritting data
-    if daily_name in ap_inst.data.columns:
+    if daily_name in ap_inst.variables:
         raise ValueError("daily Ap column name already exists: " + daily_name)
 
     # Calculate the daily mean value
@@ -209,12 +229,14 @@ def calc_daily_Ap(ap_inst, ap_name='3hr_ap', daily_name='Ap',
     # Pad the data so the first day will be backfilled
     ap_pad = pds.Series(np.full(shape=(1,), fill_value=np.nan),
                         index=[ap_mean.index[0] - pds.DateOffset(hours=3)])
+
     # Extract the mean that only uses data for one day
     ap_sel = ap_pad.combine_first(ap_mean[[i for i, tt in
                                            enumerate(ap_mean.index)
                                            if tt.hour == 21]])
+
     # Backfill this data
-    ap_data = ap_sel.resample('3H').backfill()
+    ap_data = ap_sel.resample('3H').bfill()
 
     # Save the output for the original time range
     ap_inst[daily_name] = pds.Series(ap_data[1:], index=ap_data.index[1:])
@@ -236,8 +258,9 @@ def calc_daily_Ap(ap_inst, ap_name='3hr_ap', daily_name='Ap',
     return
 
 
-def filter_geomag(inst, min_kp=0, max_kp=9, filter_time=24, kp_inst=None):
-    """Filters pysat.Instrument data for given time after Kp drops below gate.
+def filter_geomag(inst, min_kp=0, max_kp=9, filter_time=24, kp_inst=None,
+                  var_name='Kp'):
+    """Filter pysat.Instrument data for given time after Kp drops below gate.
 
     Parameters
     ----------
@@ -254,6 +277,8 @@ def filter_geomag(inst, min_kp=0, max_kp=9, filter_time=24, kp_inst=None):
     kp_inst : pysat.Instrument or NoneType
         Kp pysat.Instrument object with or without data already loaded. If None,
         will load GFZ historic kp data for the instrument date (default=None)
+    var_name : str
+        String providing the variable name for the Kp data (default='Kp')
 
     Note
     ----
@@ -267,9 +292,6 @@ def filter_geomag(inst, min_kp=0, max_kp=9, filter_time=24, kp_inst=None):
     This routine is written for standard Kp data (tag=''), not the forecast or
     recent data.  However, it will work with these Kp data if they are supplied.
 
-    This routine is designed to be used with the 'modify' flag if applied as
-    a custom routine.
-
     """
     # Load the desired data
     if kp_inst is None:
@@ -277,13 +299,21 @@ def filter_geomag(inst, min_kp=0, max_kp=9, filter_time=24, kp_inst=None):
                                    tag='', pad=pds.DateOffset(days=1))
 
     if kp_inst.empty:
-        kp_inst.load(date=inst.date, verifyPad=True)
+        load_kwargs = {'date': inst.index[0], 'end_date': inst.index[-1],
+                       'verifyPad': True}
+        if Version(pysat.__version__) > Version('3.0.1'):
+            load_kwargs['use_header'] = True
+
+        kp_inst.load(**load_kwargs)
 
     # Begin filtering, starting at the beginning of the instrument data
-    sel_data = kp_inst[(inst.date - pds.DateOffset(days=1)):
-                       (inst.date + pds.DateOffset(days=1))]
-    ind, = np.where((sel_data['Kp'] > max_kp) | (sel_data['Kp'] < min_kp))
+    sel_data = kp_inst[(inst.index[0] - pds.DateOffset(days=1)):
+                       (inst.index[-1] + pds.DateOffset(days=1))]
+    ind, = np.where((sel_data[var_name] > max_kp)
+                    | (sel_data[var_name] < min_kp))
+
     for lind in ind:
+        # Determine the time filter range for removing each flagged data
         sind = sel_data.index[lind]
         eind = sind + pds.DateOffset(hours=filter_time)
         inst[sind:eind] = np.nan
@@ -295,9 +325,8 @@ def filter_geomag(inst, min_kp=0, max_kp=9, filter_time=24, kp_inst=None):
 # --------------------------------------------------------------------------
 # Common analysis functions
 
-
-def convert_ap_to_kp(ap_data, fill_val=-1, ap_name='ap'):
-    """ Convert Ap into Kp
+def convert_ap_to_kp(ap_data, fill_val=-1, ap_name='ap', kp_name='Kp'):
+    """Convert Ap into Kp.
 
     Parameters
     ----------
@@ -306,7 +335,9 @@ def convert_ap_to_kp(ap_data, fill_val=-1, ap_name='ap'):
     fill_val : int, float, NoneType
         Fill value for the data set (default=-1)
     ap_name : str
-        Name of the input ap
+        Name of the input ap (default='ap')
+    kp_name : str
+        Name of the output Kp (default='Kp')
 
     Returns
     -------
@@ -317,60 +348,80 @@ def convert_ap_to_kp(ap_data, fill_val=-1, ap_name='ap'):
 
     """
 
-    # Ap are keys, Kp returned as double (N- = N.6667, N+=N.3333333)
-    one_third = 1.0 / 3.0
-    two_third = 2.0 / 3.0
-    ap_to_kp = {0: 0, 2: one_third, 3: two_third,
-                4: 1, 5: 1.0 + one_third, 6: 1.0 + two_third,
-                7: 2, 9: 2.0 + one_third, 12: 2.0 + two_third,
-                15: 3, 18: 3.0 + one_third, 22: 3.0 + two_third,
-                27: 4, 32: 4.0 + one_third, 39: 4.0 + two_third,
-                48: 5, 56: 5.0 + one_third, 67: 5.0 + two_third,
-                80: 6, 94: 6.0 + one_third, 111: 6.0 + two_third,
-                132: 7, 154: 7.0 + one_third, 179: 7.0 + two_third,
-                207: 8, 236: 8.0 + one_third, 300: 8.0 + two_third,
-                400: 9}
-    ap_keys = sorted([akey for akey in ap_to_kp.keys()])
-
-    # If the ap falls between two Kp indices, assign it to the lower Kp value
-    def round_ap(ap_in, fill_val=fill_val):
-        """ Round an ap value to the nearest Kp value
-        """
-        if not np.isfinite(ap_in):
-            return fill_val
-
-        i = 0
-        while ap_keys[i] <= ap_in:
-            i += 1
-        i -= 1
-
-        if i >= len(ap_keys) or ap_keys[i] > ap_in:
-            return fill_val
-
-        return ap_to_kp[ap_keys[i]]
-
     # Convert from ap to Kp
-    kp_data = np.array([ap_to_kp[aa] if aa in ap_keys else
-                        round_ap(aa, fill_val=fill_val) for aa in ap_data])
+    kp_data = np.array([round_ap(aa, fill_val=fill_val) for aa in ap_data])
 
     # Set the metadata
     meta = pysat.Meta()
-    meta['Kp'] = {meta.labels.name: 'Kp',
-                  meta.labels.desc: 'Kp converted from {:}'.format(ap_name),
-                  meta.labels.min_val: 0,
-                  meta.labels.max_val: 9,
-                  meta.labels.fill_val: fill_val,
-                  meta.labels.notes: ''.join(
-                      ['Kp converted from ', ap_name, 'as described at: ',
-                       'https://www.ngdc.noaa.gov/stp/GEOMAG/kp_ap.html'])}
+    meta[kp_name] = {meta.labels.name: 'Kp',
+                     meta.labels.desc: 'Kp converted from {:}'.format(ap_name),
+                     meta.labels.min_val: 0,
+                     meta.labels.max_val: 9,
+                     meta.labels.fill_val: fill_val,
+                     meta.labels.notes: ''.join(
+                         ['Kp converted from ', ap_name, 'as described at: ',
+                          'https://www.ngdc.noaa.gov/stp/GEOMAG/kp_ap.html'])}
 
     # Return data and metadata
     return kp_data, meta
 
 
+def round_ap(ap_in, fill_val=np.nan):
+    """Round an ap value to the nearest Kp value.
+
+    Parameters
+    ----------
+    ap_in : float
+        Ap value as a floating point number.
+    fill_val : float
+        Value for unassigned or bad indices. (default=np.nan)
+
+    Returns
+    -------
+    float
+        Fill value for infinite or out-of-range data, otherwise the best kp
+        index is provided.
+
+    """
+
+    # Define the ap to kp conversion
+    one_third = 1.0 / 3.0
+    two_third = 2.0 / 3.0
+    ap_to_kp = {0: 0, 2: one_third, 3: two_third, 4: 1, 5: 1.0 + one_third,
+                6: 1.0 + two_third, 7: 2, 9: 2.0 + one_third,
+                12: 2.0 + two_third, 15: 3, 18: 3.0 + one_third,
+                22: 3.0 + two_third, 27: 4, 32: 4.0 + one_third,
+                39: 4.0 + two_third, 48: 5, 56: 5.0 + one_third,
+                67: 5.0 + two_third, 80: 6, 94: 6.0 + one_third,
+                111: 6.0 + two_third, 132: 7, 154: 7.0 + one_third,
+                179: 7.0 + two_third, 207: 8, 236: 8.0 + one_third,
+                300: 8.0 + two_third, 400: 9}
+    max_ap = 400
+
+    # Infinite or NaN values will return the fill value
+    if not np.isfinite(ap_in):
+        return fill_val
+
+    # Ap values that correspond exactly to a Kp value will return that value
+    if ap_in in ap_to_kp.keys():
+        return ap_to_kp[ap_in]
+
+    # The input Ap value may be appropriate, but does not correspond directly
+    # to a Kp index.  Get a sorted list of directly corresponding Ap indices,
+    # with Kp returned as double (N- = N.6667, N+ = N.3333333)
+    ap_keys = sorted([akey for akey in ap_to_kp.keys() if akey <= ap_in])
+
+    # If the value is too large or too small, return the fill value
+    if len(ap_keys) == 0 or (ap_keys[-1] < ap_in and ap_keys[-1] == max_ap):
+        return fill_val
+
+    # The value is realistic, return the Kp value
+    return ap_to_kp[ap_keys[-1]]
+
+
 def combine_kp(standard_inst=None, recent_inst=None, forecast_inst=None,
                start=None, stop=None, fill_val=np.nan):
-    """ Combine the output from the different Kp sources for a range of dates
+    """Combine the output from the different Kp sources for a range of dates.
 
     Parameters
     ----------
@@ -400,6 +451,11 @@ def combine_kp(standard_inst=None, recent_inst=None, forecast_inst=None,
         time, merging the standard, recent, and forecasted values based on
         their reliability
 
+    Raises
+    ------
+    ValueError
+        If only one Kp instrument or bad times are provided
+
     Note
     ----
     Merging prioritizes the standard data, then the recent data, and finally
@@ -443,8 +499,7 @@ def combine_kp(standard_inst=None, recent_inst=None, forecast_inst=None,
 
     if stop is None:
         stimes = [inst.index.max() for inst in all_inst if len(inst.index) > 0]
-        stop = max(stimes) if len(stimes) > 0 else None
-        stop += pds.DateOffset(days=1)
+        stop = max(stimes) + dt.timedelta(days=1) if len(stimes) > 0 else None
 
     if start is None or stop is None:
         raise ValueError(' '.join(("must either load in Instrument objects or",
@@ -467,7 +522,11 @@ def combine_kp(standard_inst=None, recent_inst=None, forecast_inst=None,
     while itime < stop and inst_flag is not None:
         # Load and save the standard data for as many times as possible
         if inst_flag == 'standard':
-            standard_inst.load(date=itime)
+            load_kwargs = {'date': itime}
+            if Version(pysat.__version__) > Version('3.0.1'):
+                load_kwargs['use_header'] = True
+
+            standard_inst.load(**load_kwargs)
 
             if notes.find("standard") < 0:
                 notes += " the {:} source ({:} to ".format(inst_flag,
@@ -493,7 +552,10 @@ def combine_kp(standard_inst=None, recent_inst=None, forecast_inst=None,
             # data
             for filename in files:
                 if filename is not None:
-                    recent_inst.load(fname=filename)
+                    load_kwargs = {'fname': filename}
+                    if Version(pysat.__version__) > Version('3.0.1'):
+                        load_kwargs['use_header'] = True
+                    recent_inst.load(**load_kwargs)
 
                 if notes.find("recent") < 0:
                     notes += " the {:} source ({:} to ".format(inst_flag,
@@ -526,7 +588,10 @@ def combine_kp(standard_inst=None, recent_inst=None, forecast_inst=None,
             # data
             for filename in files:
                 if filename is not None:
-                    forecast_inst.load(fname=filename)
+                    load_kwargs = {'fname': filename}
+                    if Version(pysat.__version__) > Version('3.0.1'):
+                        load_kwargs['use_header'] = True
+                    forecast_inst.load(**load_kwargs)
 
                 if notes.find("forecast") < 0:
                     notes += " the {:} source ({:} to ".format(inst_flag,
@@ -553,7 +618,6 @@ def combine_kp(standard_inst=None, recent_inst=None, forecast_inst=None,
         notes += "{:})".format(itime.date())
 
     # Determine if the beginning or end of the time series needs to be padded
-
     freq = None if len(kp_times) < 2 else pysat.utils.time.calc_freq(kp_times)
     end_date = stop - pds.DateOffset(days=1)
     date_range = pds.date_range(start=start, end=end_date, freq=freq)
